from django.dispatch import Signal
from django.db import reset_queries, close_old_connections


message_started = Signal()
message_finished = Signal()

# db connection state managed similarly to the wsgi handler
message_started.connect(reset_queries)
message_started.connect(close_old_connections)
message_finished.connect(close_old_connections)



import json

import amqp

from .connection_params import ParamsFactory
from .connection import EDAConnection


class EDAPublisher:
    def __init__(self, connection_params_factory: ParamsFactory):
        self._connection_params = connection_params_factory.get_params()

    def send_message(self, body: dict, exchange: str, routing_key: str = ""):
        connection = EDAConnection.get_connection(self._connection_params)
        channel = connection.channel()

        message = amqp.Message(json.dumps(body))

        channel.basic_publish(exchange=exchange, msg=message, routing_key=routing_key)



from abc import ABC, abstractmethod

import amqp

from .signals import message_started, message_finished


class EDAConsumer(ABC):  # pragma: no cover
    _message: amqp.Message

    def handle(self, message: amqp.Message):
        self._message = message

        message_started.send(sender=self)
        try:
            self.consume(message)
        except Exception as exception:
            message.channel.basic_reject(message.delivery_tag, requeue=False)
            print(f"[{self.__class__.__name__}] - Message rejected by: {exception}")
        finally:
            message_finished.send(sender=self)

    def ack(self):
        self._message.channel.basic_ack(self._message.delivery_tag)

    @abstractmethod
    def consume(self, message: amqp.Message):
        pass



import json

from .parser import Parser
from .exceptions import ParseError


class JSONParser(Parser):
    @staticmethod
    def parse(stream, encoding="utf-8"):
        """
        Parses the incoming bytestream as JSON and returns the resulting data.
        """

        if not stream:
            raise ParseError("JSON parse error - stream cannot be empty")

        try:
            decoded_stream = stream.decode(encoding)
            return json.loads(decoded_stream)
        except ValueError as exc:
            raise ParseError("JSON parse error - %s" % str(exc))



from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.module_loading import import_string

from weni.eda.backends.pyamqp_backend import PyAMQPConnectionBackend
from weni.eda.django.connection_params import ConnectionParamsFactory


handle_consumers_function = import_string(settings.EDA_CONSUMERS_HANDLE)


def get_connection_backend():  # pragma: no cover
    if hasattr(settings, "EDA_CONNECTION_BACKEND"):
        return import_string(settings.EDA_CONNECTION_BACKEND)(handle_consumers_function)

    return PyAMQPConnectionBackend(handle_consumers_function)


class Command(BaseCommand):  # pragma: no cover
    def handle(self, *args, **options):
        connection_params = ConnectionParamsFactory.get_params()
        get_connection_backend().start_consuming(connection_params)



import amqp
from .connection_params import ConnectionParams


class EDAConnection:
    connection: amqp.Connection = None

    @classmethod
    def get_connection(cls, connection_params: ConnectionParams) -> amqp.Connection:
        if cls.connection is None or not cls.connection.connected:
            cls.connection = amqp.Connection(**connection_params.value)
            cls.connection.connect()

        return cls.connection



from unittest import TestCase

from ..json_parser import JSONParser
from ..exceptions import ParseError


class JSONParserTestCase(TestCase):
    def test_parse_invalid_stream_format(self):
        stream = b"abcd"

        with self.assertRaises(ParseError):
            JSONParser().parse(stream)

    def test_parse_empty_stream(self):
        stream = b""

        with self.assertRaises(ParseError):
            JSONParser().parse(stream)

    def test_parse_invalid_encoding(self):
        stream = b'{\r\n    "code": "123"\r\n}'

        with self.assertRaises(LookupError):
            JSONParser().parse(stream, encoding="123")

    def test_parse_valid_json_and_encoding(self):
        stream = b'{\r\n    "code": "123"\r\n}'
        json_parsed = JSONParser().parse(stream)

        self.assertEqual(json_parsed, {"code": "123"})
